-- YGREC8/INC16.vhdl
-- created sam. nov. 11 18:50:10 CET 2017 by Yann Guidon (whygee@f-cpu.org)
-- version dim. nov. 12 05:44:22 CET 2017
-- Released under the GNU AGPLv3 license
--
-- This is the increment unit of YGREC8,
-- used to increment the timer.
-- This high speed version uses 3-input gates
-- from Actel/Microsemi's A3P FPGA family.
-- A couple of interleaved AND trees keep fanout low.

Library ieee;
    use ieee.std_logic_1164.all;

entity INC16 is
  port(
    A : in  std_logic_vector(15 downto 0);
    Y : out std_logic_vector(15 downto 0);
    C : out std_logic_vector(15 downto 0);
    V : out std_logic);
end INC16;

architecture A3P of INC16 is
  Signal
    -- L1:
    A01,  A012, A234, A345, A567,  -- notice there is some overlap
    A678, A89A, A9AB, ABCD,
    -- L2:
    A05, A07, A08, A8C, A8D, A9E : std_logic;
  Signal CC: std_logic_vector(15 downto 0);

begin

  -- Fanin :
  -- A( 0) : 3
  -- A( 1) : 2
  -- A( 2) : 2
  -- A( 3) : 3
  -- A( 4) : 2
  -- A( 5) : 2
  -- A( 6) : 3
  -- A( 7) : 2
  -- A( 8) : 2
  -- A( 9) : 3
  -- A(10) : 2
  -- A(11) : 3
  -- A(12) : 2
  -- A(13) : 1
  -- A(14) : 1
  -- A(15) : 1

  -- Layer 1 : 
   A01 <= A( 0) and A( 1);           -- FO3
  A012 <= A( 0) and A( 1) and A( 2); -- FO4
  A234 <= A( 2) and A( 3) and A( 4); -- FO2
  A345 <= A( 3) and A( 4) and A( 5); -- FO2
  A567 <= A( 5) and A( 6) and A( 7); -- FO1
  A678 <= A( 6) and A( 7) and A( 8); -- FO1
  A89A <= A( 8) and A( 9) and A(10); -- FO3
  A9AB <= A( 9) and A(10) and A(11); -- FO2
  ABCD <= A(11) and A(12) and A(13); -- FO2

  -- layer 2
  A05 <= A012 and A345;            -- FO2
  A07 <= A01  and A234 and A567;   -- FO4
  A08 <= A012 and A345 and A678;   -- FO5
  A8C <= A89A and A(11) and A(12); -- FO1
  A8D <= A89A and ABCD;            -- FO1
  A9E <= A9AB and ABCD and A(14);  -- FO2

  -- the CC layer can be only AND2 so they are combined with XOR
  CC( 0) <= '1';
  CC( 1) <= A(0);                  -- L0
  CC( 2) <= A01;                   -- L1
  CC( 3) <= A012;                  -- L1
  CC( 4) <= A012 and A(3);         -- L2
  CC( 5) <= A01  and A234;         -- L2
  CC( 6) <= A05;                   -- L2
  CC( 7) <= A05 and A(6);          -- L3
  CC( 8) <= A07;                   -- L3
  CC( 9) <= A08;                   -- L2
  CC(10) <= A08 and A(9);          -- L3
  CC(11) <= A07 and A89A;          -- L3
  CC(12) <= A08 and A9AB;          -- L3
  CC(13) <= A07 and A8C;           -- L3
  CC(14) <= A07 and A8D;           -- L3
  CC(15) <= A08 and A9E;           -- L3
  V      <= A08 and A9E and A(15); -- L3 (no XOR here, AND3)
  C      <= CC;
  Y      <= A xor CC;

end A3P;
