-- YGREC8/REG8_A3P.vhdl
-- created ven. nov. 10 22:02:40 CET 2017 by Yann Guidon (whygee@f-cpu.org)
-- version sam. nov. 11 00:48:04 CET 2017
-- version lun. nov. 13 02:24:37 CET 2017
-- Released under the GNU AGPLv3 license
--
-- This is the register set of the YGREC8, without the PC.
-- This version is designed for ACTEL/Microsemi A3P FPGA
-- using a modified binary tree with extra shuffling.

Library ieee;
    use ieee.std_logic_1164.all;
Library work;
    use work.all;
    use work.ygrec8_def.all;

entity REG8 is
  port( clk, reset, write_enable : in std_logic;
    AdrSRC, AdrDST : in std_logic_vector(2 downto 0);
    PC, D1, D2, RESULT : in std_logic_vector(7 downto 0);
    SRC, DST, A1, A2  : out std_logic_vector(7 downto 0));
end REG8;

architecture A3P_2 of REG8 is
  Signal tA1, tA2, R1, R2, R3 : std_logic_vector(7 downto 0);
  Signal LS1a_x, LS1b_x, LS1c_x, LS1d_x, LS2a_x, LS2b_x, LS3_x : std_logic_vector(2 downto 0);
  Signal LS1a_y, LS1b_y, LS1c_y, LS1d_y, LS2a_y, LS2b_y, LS3_y : std_logic_vector(2 downto 0);
  Signal LS1a_z, LS1b_z, LS1c_z, LS1d_z, LS2a_z, LS2b_z, LS3_z : std_logic_vector(1 downto 0);
  Signal LD1a_x, LD1b_x, LD1c_x, LD1d_x, LD2a_x, LD2b_x, LD3_x : std_logic_vector(2 downto 0);
  Signal LD1a_y, LD1b_y, LD1c_y, LD1d_y, LD2a_y, LD2b_y, LD3_y : std_logic_vector(2 downto 0);
  Signal LD1a_z, LD1b_z, LD1c_z, LD1d_z, LD2a_z, LD2b_z, LD3_z : std_logic_vector(1 downto 0);

begin

  A1 <= tA1;
  A2 <= tA2; -- temp registers because mode inout is pretty annoying

  -- Read the registers with super-balanced MUX trees:
  SRC  <= LS3_z & LS3_y & LS3_x;

  -- fanout 0:9  1:9  2:3
  LS3_x  <= LS2a_x when AdrSRC(2)='0' else LS2b_x;
  LS2a_x <= LS1a_x when AdrSRC(1)='0' else LS1b_x;  -- lower half
  LS2b_x <= LS1c_x when AdrSRC(0)='0' else LS1d_x;  -- higher half
  LS1a_x <=  D1(2 downto 0) when AdrSRC(0)='0' else tA1(2 downto 0);  -- 0, 1
  LS1b_x <=  D2(2 downto 0) when AdrSRC(0)='0' else tA2(2 downto 0);  -- 2, 3
  LS1c_x <=  R1(2 downto 0) when AdrSRC(1)='0' else  R3(2 downto 0);  -- 4, 6
  LS1d_x <=  R2(2 downto 0) when AdrSRC(1)='0' else  PC(2 downto 0);  -- 5, 7

  -- fanout 0:9  1:3  2:9
  LS3_y  <= LS2a_y when AdrSRC(1)='0' else LS2b_y;
  LS2a_y <= LS1a_y when AdrSRC(2)='0' else LS1b_y; --  0, 1, 4, 5
  LS2b_y <= LS1c_y when AdrSRC(0)='0' else LS1d_y; --  2, 3, 6, 7
  LS1a_y <=  D1(5 downto 3) when AdrSRC(0)='0' else tA1(5 downto 3);  -- 0, 1
  LS1b_y <=  R1(5 downto 3) when AdrSRC(0)='0' else  R2(5 downto 3);  -- 4, 5
  LS1c_y <=  D2(5 downto 3) when AdrSRC(2)='0' else  R3(5 downto 3);  -- 2, 6
  LS1d_y <= tA2(5 downto 3) when AdrSRC(2)='0' else  PC(5 downto 3);  -- 3, 7

  -- fanout 0:2  1:6  2:6
  LS3_z  <= LS2a_z when AdrSRC(0)='0' else LS2b_z;
  LS2a_z <= LS1a_z when AdrSRC(1)='0' else LS1b_z;  -- even number
  LS2b_z <= LS1c_z when AdrSRC(2)='0' else LS1d_z;  -- odd numbers
  LS1a_z <=  D1(7 downto 6) when AdrSRC(2)='0' else  R1(7 downto 6);  -- 0, 4
  LS1b_z <=  D2(7 downto 6) when AdrSRC(2)='0' else  R3(7 downto 6);  -- 2, 6
  LS1c_z <= tA1(7 downto 6) when AdrSRC(1)='0' else tA2(7 downto 6);  -- 1, 3
  LS1d_z <=  R2(7 downto 6) when AdrSRC(1)='0' else  PC(7 downto 6);  -- 5, 7

  -- total fanout : AdrSRC(0):20, AdrSRC(1):18, AdrSRC(2): 18
  -- individual fanout is almost half of max=32 and close to (7x8=56)/3=18.66

  DST  <= LD3_z & LD3_y & LD3_x;

  LD3_x  <= LD2a_x when AdrDST(2)='0' else LD2b_x;
  LD2a_x <= LD1a_x when AdrDST(1)='0' else LD1b_x;  -- lower half
  LD2b_x <= LD1c_x when AdrDST(0)='0' else LD1d_x;  -- higher half
  LD1a_x <=  D1(2 downto 0) when AdrDST(0)='0' else tA1(2 downto 0);  -- 0, 1
  LD1b_x <=  D2(2 downto 0) when AdrDST(0)='0' else tA2(2 downto 0);  -- 2, 3
  LD1c_x <=  R1(2 downto 0) when AdrDST(1)='0' else  R3(2 downto 0);  -- 4, 6
  LD1d_x <=  R2(2 downto 0) when AdrDST(1)='0' else  PC(2 downto 0);  -- 5, 7

  LD3_y  <= LD2a_y when AdrDST(1)='0' else LD2b_y;
  LD2a_y <= LD1a_y when AdrDST(2)='0' else LD1b_y; --  0, 1, 4, 5
  LD2b_y <= LD1c_y when AdrDST(0)='0' else LD1d_y; --  2, 3, 6, 7
  LD1a_y <=  D1(5 downto 3) when AdrDST(0)='0' else tA1(5 downto 3);  -- 0, 1
  LD1b_y <=  R1(5 downto 3) when AdrDST(0)='0' else  R2(5 downto 3);  -- 4, 5
  LD1c_y <=  D2(5 downto 3) when AdrDST(2)='0' else  R3(5 downto 3);  -- 2, 6
  LD1d_y <= tA2(5 downto 3) when AdrDST(2)='0' else  PC(5 downto 3);  -- 3, 7

  LD3_z  <= LD2a_z when AdrDST(0)='0' else LD2b_z;
  LD2a_z <= LD1a_z when AdrDST(1)='0' else LD1b_z;  -- even number
  LD2b_z <= LD1c_z when AdrDST(2)='0' else LD1d_z;  -- odd numbers
  LD1a_z <=  D1(7 downto 6) when AdrDST(2)='0' else  R1(7 downto 6);  -- 0, 4
  LD1b_z <=  D2(7 downto 6) when AdrDST(2)='0' else  R3(7 downto 6);  -- 2, 6
  LD1c_z <= tA1(7 downto 6) when AdrDST(1)='0' else tA2(7 downto 6);  -- 1, 3
  LD1d_z <=  R2(7 downto 6) when AdrDST(1)='0' else  PC(7 downto 6);  -- 5, 7

  -- RESET and WRITE:
  process (clk, reset)
  begin
    if reset='0' then
      tA1 <= "00000000";  -- Note :
      tA2 <= "00000000";  -- Reset does	not necessarily	       
      R1  <= "00000000";  -- clear the registers, only PC !
      R2  <= "00000000";  -- Never assume a reset value
      R3  <= "00000000";  -- (these are	costs-saving tricks)
    else
      if rising_edge(clk) and write_enable='1' then
        case AdrDST is
          when Reg_A1 => tA1 <= RESULT; -- report "write to tA1";
          when Reg_A2 => tA2 <= RESULT; -- report "write to tA2";
          when Reg_R1 =>  R1 <= RESULT; -- report "write to  R1";
          when Reg_R2 =>  R2 <= RESULT; -- report "write to  R2";
          when Reg_R3 =>  R3 <= RESULT; -- report "write to  R3";
          when others => -- PC, D1, D2 are stored somewhere else
        end case;
      end if;
    end if;
  end process;
end A3P_2;

