-- YGREC8/REG8_tb.vhdl
-- version lun. nov. 13 01:44:55 CET 2017 by Yann Guidon (whygee@f-cpu.org)
-- Released under the GNU AGPLv3 license

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
Library work;
    use work.all;
    use work.ygrec8_def.all;
    use work.ygrec8_utils.all;

entity REG8_tb is
end REG8_tb;

architecture plip of REG8_tb is
  signal clk, reset, write_enable : std_logic := '0';
  signal AdrSRC, AdrDST : std_logic_vector(2 downto 0);
  signal PC, D1, D2, RESULT,
    SRC, DST, A1, A2  : std_logic_vector(7 downto 0) := "00000000";

begin

  tb: entity REG8 port map (
    clk => clk,
    reset => reset,
    write_enable => write_enable,
    AdrSRC => AdrSRC,
    AdrDST => AdrDST,
    PC => PC,
    D1 => D1,
    D2 => D2,
    RESULT => RESULT,
    SRC => SRC,
    DST => DST,
    A1 => A1,
    A2 => A2
  );

  bench: process
    variable i : integer := 0;
    variable t, u : std_logic_vector(7 downto 0);

    procedure ShowRegisters is
      variable i : integer := 0;
      variable Adr : std_logic_vector(2 downto 0);
    begin
      report "* REGISTERS:";
      for i in 0 to 7 loop
        Adr := std_logic_vector(to_unsigned(i, 3));
        AdrSRC <= Adr;
        AdrDST <= Adr;
        wait for 1 ns;

	report "@" & SLV_to_bin(Adr)
           & " SRC=" & SLV_to_bin(SRC)
           & " DST=" & SLV_to_bin(DST);

      end loop;
    end ShowRegisters;

    procedure WriteReg(nr : integer; val: std_logic_vector(7 downto 0)) is
      variable Adr : std_logic_vector(2 downto 0);
    begin
      Adr := std_logic_vector(to_unsigned(i, 3));
      AdrSRC <= Adr;
      AdrDST <= Adr;
      RESULT <= val;

      case nr is
        when 0 => D1 <= val; -- report "write to D1";
        when 2 => D2 <= val; -- report "write to D2";
        when 7 => PC <= val; -- report "write to PC";
        when others => -- the rest is stored inside the RegSet
      end case;

      wait for 1 ns;
      clk <= '1';
      wait for 1 ns;
      clk <= '0';

      if SRC /= val then
        report "Reg#" & integer'image(nr)
             & "  SRC=" & SLV_to_bin(SRC)
             & "  expected=" & SLV_to_bin(val);
      end if;
      if DST /= val then
        report "Reg#" & integer'image(nr)
             & "  DST=" & SLV_to_bin(DST)
             & "  expected=" & SLV_to_bin(val);
      end if;

    end WriteReg;

  begin
    -- reset
    D1 <= "00000000";
    D2 <= "00000000";
    PC <= "00000000";
    clk <= '0';
    reset <= '0';
    write_enable <= '0';
    wait for 1 ns;

    reset <= '1';
    write_enable <= '1';
    wait for 1 ns;
--    ShowRegisters;

    for i in 0 to 7 loop
      WriteReg(i, "11111111");
--      ShowRegisters;
    end loop;
 
    wait;
  end process;

end plip;
