-- YGREC8/SHL8_tb.vhdl
-- version lun. nov. 13 01:46:12 CET 2017 by Yann Guidon (whygee@f-cpu.org)
-- Released under the GNU AGPLv3 license

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
Library work;
    use work.all;
    use work.ygrec8_def.all;
    use work.ygrec8_utils.all;

entity SHL8_tb is
end SHL8_tb;

architecture plip of SHL8_tb is
  signal F : std_logic_vector(3 downto 0) := "0000";
  signal SRC, DST, SHL : std_logic_vector(7 downto 0) := "00000000";
begin

  tb: entity SHL8 port map (
     F0 => F(0),
     F1 => F(1),
     F2 => F(2),
     F3 => F(3),
     SRC => SRC,
     DST => DST,
     SHL => SHL);

  bench: process
    variable i, j : integer := 0;

    procedure TestVector (i : integer) is
      variable KO: integer := 0;
      variable expected, R1, R2, R3, R4 : std_logic_vector(7 downto 0);
      variable t : std_logic_vector(15 downto 0);
    begin
      F <= OP_SHR; wait for 1 ns; R1 := SHL;  -- SHR = "1000"
         expected := std_logic_vector(unsigned(DST) srl i);
         if expected /= SHL then KO:=KO+1; end if;

      F <= OP_SHL; wait for 1 ns; R2 := SHL;  -- SHL = "1001"
         expected := std_logic_vector(unsigned(DST) sll i);
         if expected /= SHL then KO:=KO+1; end if;

      F <= Op_SAR; wait for 1 ns; R3 := SHL;  -- SAR = "1010"
         expected := (others => DST(7)); -- sign extension
         t := std_logic_vector(unsigned(expected & DST) srl i);
         expected := t(7 downto 0);
         if expected /= SHL then KO:=KO+1; end if;

      F <= Op_ROL; wait for 1 ns; R4 := SHL;  -- ROL = "1011"
         expected := std_logic_vector(unsigned(DST) rol i);
         if expected /= SHL then KO:=KO+1; end if;

      if KO /= 0 then
        report    SLV_to_bin(DST)
          & " " & SLV_to_bin(SRC)
          & " " & SLV_to_bin( R1)
          & " " & SLV_to_bin( R2)
          & " " & SLV_to_bin( R3)
          & " " & SLV_to_bin( R4);
      end if;
    end TestVector;

  begin
    report "Testing SHL:";
    report "   DST      SRC      SHR      SHL      SAR       ROL";
    for i in 0 to 7 loop
      SRC <= std_logic_vector(to_unsigned(i, 8));

      for j in 0 to 255 loop
        DST <= std_logic_vector(to_unsigned(j, 8));

        TestVector(i);
      end loop;
    end loop;

    wait;
  end process;

end plip;
