-- YGREC8/YGREC8_utils.vhdl
-- created lun. nov. 13 01:02:38 CET 2017 by Yann Guidon (whygee@f-cpu.org)
-- Released under the GNU AGPLv3 license
--
-- These are some useful functions for the YGREC8,
-- shamelessly "borrowed" from yasep_utils.vhdl

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;

package ygrec8_utils is
  function safe_to_integer(u:std_logic_vector) return integer;
  function SLV_to_bin(s:std_logic_vector) return string;
  function SLV4_to_Hex(s:std_logic_vector(3 downto 0)) return character;
  function SLV_to_Hex(s:std_logic_vector) return string;
end ygrec8_utils;

package body ygrec8_utils is

  function safe_to_integer(u:std_logic_vector) return integer is
  begin
   if Is_X(u) then
      return 0;
    else
      return to_integer(unsigned(u));
    end if;
  end;

  function SLV_to_bin(s:std_logic_vector) return string is
    variable t : string(s'range);
    variable u : string(3 downto 1);
  begin
    for i in s'range loop
      u := std_logic'image(s(i));
      t(i) := u(2);
    end loop;
    return t;
  end SLV_to_bin;

  function SLV4_to_Hex(s: std_logic_vector(3 downto 0)) return character is
  begin
    case s is
      when "0000" => return '0';
      when "0001" => return '1';
      when "0010" => return '2';
      when "0011" => return '3';
      when "0100" => return '4';
      when "0101" => return '5';
      when "0110" => return '6';
      when "0111" => return '7';
      when "1000" => return '8';
      when "1001" => return '9';
      when "1010" => return 'A';
      when "1011" => return 'B';
      when "1100" => return 'C';
      when "1101" => return 'D';
      when "1110" => return 'E';
      when "1111" => return 'F';
      when others => return '?';
    end case;
  end;

  function SLV_to_Hex(s:std_logic_vector) return string is
    variable u: std_logic_vector(s'length+3 downto 0);
    variable t: string((s'length-1)/4 downto 0);
    variable i: integer:= 0;
    variable j: integer:= 0;
  begin
    u:= "0000" & s;
    while (i < s'length) loop
      t(j) := SLV4_to_Hex(u(i+3 downto i));
      i := i+4;
      j := j+1;
    end loop;
    return t;
  end;

end ygrec8_utils;
